@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.21 2007/02/03 04:56:18 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/child.c,v 1.23 2007/05/17 02:05:50 yamaguti Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -259,7 +259,7 @@ void do_child(int unix_fd, int inet_fd)
 
 		/* look for existing connection */
 		found = 0;
-		backend = pool_get_cp(sp->user, sp->database, sp->major);
+		backend = pool_get_cp(sp->user, sp->database, sp->major, 1);
 
 		if (backend != NULL)
 		{
@@ -365,7 +365,7 @@ void do_child(int unix_fd, int inet_fd)
 		/* show ps status */
 		sp = MASTER_CONNECTION(backend)->sp;
 		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle",
-				 sp->database, sp->user, remote_ps_data);
+				 sp->user, sp->database, remote_ps_data);
 		set_ps_display(psbuf, false);
 
 		/* query process loop */
