@@ -1,12 +1,12 @@
 /* -*-pgsql-c-*- */
 /*
  *
- * $Header: /cvsroot/pgpool/pgpool/pool.h,v 1.11 2006/08/30 01:11:47 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool.h,v 1.17 2007/02/12 06:21:03 y-asaba Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
  *
- * Copyright (c) 2003-2006	PgPool Global Development Group
+ * Copyright (c) 2003-2007	PgPool Global Development Group
  *
  * Permission to use, copy, modify, and distribute this software and
  * its documentation for any purpose and without fee is hereby
@@ -28,8 +28,12 @@
 
 #include "config.h"
 #include "pool_signal.h"
+#include "pool_type.h"
+#include "pool_list.h"
+
 #include <stdio.h>
 #include <time.h>
+#include <netinet/in.h>
 
 /* undef this if you have problems with non blocking accept() */
 #define NONE_BLOCK
@@ -38,6 +42,7 @@
 
 /* configuration file name */
 #define POOL_CONF_FILE_NAME "pgpool.conf"
+#define HBA_CONF_FILE_NAME "pool_hba.conf"
 
 /* pid file directory */
 #define DEFAULT_LOGDIR "/tmp"
@@ -159,6 +164,9 @@ typedef struct {
 	int num_servers;			/* number of PostgreSQL servers */
 	int server_status[MAX_CONNECTION_SLOTS];	/* server status 0:unused, 1:up, 2:down */
 	int log_statement; /* 0:false, 1: true - logs all SQL statements */
+	int log_connections;		/* 0:false, 1:true - logs incoming connections */
+	int log_hostname;		/* 0:false, 1:true - resolve hostname */
+	int enable_pool_hba;		/* 0:false, 1:true - enables pool_hba.conf file authentication */
 } POOL_CONFIG;
 
 #define MAX_PASSWORD_SIZE		1024
@@ -211,6 +219,19 @@ typedef struct {
 
 	int no_forward;		/* if non 0, do not write to frontend */
 
+	/*
+	 * frontend info needed for hba
+	 */
+	int protoVersion;
+	SockAddr raddr;
+	UserAuth auth_method;
+	char *auth_arg;
+	char *database;
+	char *username;
+#ifdef USE_SSL
+	bool ssl;
+#endif
+
 } POOL_CONNECTION;
 
 /*
@@ -255,6 +276,7 @@ typedef struct {
 extern POOL_CONFIG pool_config;	/* configuration values */
 extern POOL_CONNECTION_POOL *pool_connection_pool;	/* connection pool */
 extern long int weight_master;	/* normalized weight of master (0-RAND_MAX range) */
+extern char remote_ps_data[];		/* used for set_ps_display */
 
 /*
  * public functions
@@ -285,7 +307,7 @@ extern int pool_do_reauth(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *cp);
 
 extern int pool_init_cp(void);
 extern POOL_CONNECTION_POOL *pool_create_cp(void);
-extern POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor);
+extern POOL_CONNECTION_POOL *pool_get_cp(char *user, char *database, int protoMajor, int check_socket);
 extern void pool_discard_cp(char *user, char *database, int protoMajor);
 
 extern POOL_STATUS ErrorResponse(POOL_CONNECTION *frontend, 
@@ -339,4 +361,24 @@ extern int health_check(void);
 
 extern void init_prepared_list(void);
 
+extern size_t strlcpy(char *dst, const char *src, size_t siz);
+
+extern bool update_process_title;
+
+extern char **save_ps_display_args(int argc, char **argv);
+
+extern void init_ps_display(const char *username, const char *dbname,
+							const char *host_info, const char *initial_str);
+
+extern void set_ps_display(const char *activity, bool force);
+
+extern const char *get_ps_display(int *displen);
+
+/* pool_hba.c */
+extern void load_hba(char *hbapath);
+extern void ClientAuthentication(POOL_CONNECTION *frontend);
+
+/* pool_ip.c */
+extern void pool_getnameinfo_all(SockAddr *saddr, char *remote_host, char *remote_port);
+
 #endif /* POOL_H */
