@@ -1,6 +1,6 @@
 /* -*-pgsql-c-*- */
 /*
- * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.41 2007/02/03 04:22:21 t-ishii Exp $
+ * $Header: /cvsroot/pgpool/pgpool/pool_process_query.c,v 1.45 2007/05/17 02:05:50 yamaguti Exp $
  *
  * pgpool: a language independent connection pool server for PostgreSQL 
  * written by Tatsuo Ishii
@@ -115,6 +115,8 @@ static int synchronize(POOL_CONNECTION *cp);
 static void process_reporting(POOL_CONNECTION *frontend, POOL_CONNECTION_POOL *backend);
 static int reset_backend(POOL_CONNECTION_POOL *backend, int qcnt);
 
+static int is_select_query(char *sql);
+static int is_sequence_query(char *sql);
 static int load_balance_enabled(POOL_CONNECTION_POOL *backend, char *sql);
 static void start_load_balance(POOL_CONNECTION_POOL *backend);
 static void end_load_balance(POOL_CONNECTION_POOL *backend);
@@ -143,6 +145,7 @@ static int replication_was_enabled;		/* replication mode was enabled */
 static int master_slave_was_enabled;	/* master/slave mode was enabled */
 static int internal_transaction_started;		/* to issue table lock command a transaction
 												   has been started internally */
+static int select_in_transaction = 0; /* non 0 if select query is in transaction */
 static void (*pending_function)(PreparedStatementList *p, PreparedStatement *statement) = NULL;
 static PreparedStatement *pending_prepared_stmt = NULL;
 
@@ -634,7 +637,7 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 		/* show ps status */
 		sp = MASTER_CONNECTION(backend)->sp;
 		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle",
-				 sp->database, sp->user, remote_ps_data);
+				 sp->user, sp->database, remote_ps_data);
 		set_ps_display(psbuf, false);
 		return POOL_CONTINUE;
 	}
@@ -645,13 +648,12 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 		PreparedStatement *stmt;
 
 		stmt = get_prepared_command_portal_and_statement(string);
-		/* could not get info. probably wrong SQL command */
-		if (stmt == NULL)
+
+		if (stmt != NULL)
 		{
-			return POOL_CONTINUE;
+			pending_function = add_prepared_list;
+			pending_prepared_stmt = stmt;
 		}
-		pending_function = add_prepared_list;
-		pending_prepared_stmt = stmt;
 	}
 	else if (frontend &&
 			 strncasecmp("deallocate", string, 10) == 0)
@@ -695,19 +697,20 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 		char *portal_name = get_execute_command_portal_name(string);
 		PreparedStatement *stmt;
 
-		/* could not get portal name. probably wrong SQL command */
-		if (portal_name == NULL)
+		if (portal_name != NULL)
 		{
-			return POOL_CONTINUE;
-		}
-
-		stmt = lookup_prepared_statement_by_statement(&prepared_list,
-													  portal_name);
+			stmt = lookup_prepared_statement_by_statement(&prepared_list,
+														  portal_name);
 		
-		if (!stmt)
-			string1 = string;
+			if (!stmt)
+				string1 = string;
+			else
+				string1 = stmt->prepared_string;
+		}
 		else
-			string1 = stmt->prepared_string;
+		{
+			string1 = string;
+		}
 	}
 	else
 	{
@@ -723,6 +726,21 @@ static POOL_STATUS Query(POOL_CONNECTION *frontend,
 		MASTER_SLAVE = 0;
 		master_slave_dml = 1;
 	}
+	else if (REPLICATION && is_select_query(string1) && !is_sequence_query(string1))
+	{
+		int i;
+		/* save backend connection slots */
+		for (i=0;i<backend->num;i++)
+		{
+			slots[i] = backend->slots[i];
+		}
+
+		/* send query to master only. */
+		replication_was_enabled = 1;
+		REPLICATION = 0;
+		in_load_balance = 1;
+		select_in_transaction = 1;
+	}
 
 	/*
 	 * judge if we need to lock the table
@@ -842,6 +860,22 @@ static POOL_STATUS Execute(POOL_CONNECTION *frontend,
 		MASTER_SLAVE = 0;
 		master_slave_dml = 1;
 	}
+	else if (REPLICATION && is_select_query(stmt->prepared_string) && !is_sequence_query(stmt->prepared_string))
+	{
+		int i;
+
+		/* save backend connection slots */
+		for (i=0;i<backend->num;i++)
+		{
+			slots[i] = backend->slots[i];
+		}
+
+		/* send query to master only. */
+		replication_was_enabled = 1;
+		REPLICATION = 0;
+		in_load_balance = 1;
+		select_in_transaction = 1;
+	}
 
 	for (i = 0;i < backend->num;i++)
 	{
@@ -1057,10 +1091,10 @@ static POOL_STATUS ReadyForQuery(POOL_CONNECTION *frontend,
 	sp = MASTER_CONNECTION(backend)->sp;
 	if (MASTER(backend)->tstate == 'T')
 		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle in transaction", 
-				 sp->database, sp->user, remote_ps_data);
+				 sp->user, sp->database, remote_ps_data);
 	else
 		snprintf(psbuf, sizeof(psbuf), "%s %s %s idle", 
-				 sp->database, sp->user, remote_ps_data);
+				 sp->user, sp->database, remote_ps_data);
 	set_ps_display(psbuf, false);
 
 	return POOL_CONTINUE;
@@ -2501,6 +2535,8 @@ POOL_STATUS SimpleForwardToFrontend(char kind, POOL_CONNECTION *frontend, POOL_C
 	{
 		pending_function(&prepared_list, pending_prepared_stmt);
 	}
+	else if (kind == 'C' && select_in_transaction)
+		select_in_transaction = 0;
 
 	pending_function = NULL;
 	pending_prepared_stmt = NULL;
@@ -2588,6 +2624,19 @@ POOL_STATUS SimpleForwardToFrontend(char kind, POOL_CONNECTION *frontend, POOL_C
 			}
 		}
 
+		if (select_in_transaction)
+		{
+			do_command(SECONDARY(backend), "send invalid query from pgpool to abort transaction.",
+					   PROTO_MAJOR_V3, 0);
+			pool_write(SECONDARY(backend), "S", 1);
+			res1 = htonl(4);
+			if (pool_write_and_flush(SECONDARY(backend), &res1, sizeof(res1)) < 0)
+			{
+				return POOL_END;
+			}
+			select_in_transaction = 0;
+		}
+
 		for (i = 0;i < backend->num;i++)
 		{
 			POOL_CONNECTION *cp = backend->slots[i]->con;
@@ -2694,10 +2743,10 @@ POOL_STATUS SimpleForwardToBackend(char kind, POOL_CONNECTION *frontend, POOL_CO
 	if (p == NULL)
 		return POOL_END;
 
-	if (pool_write_and_flush(MASTER(backend), p, len))
+	if (pool_write(MASTER(backend), p, len))
 		return POOL_END;
 	if (REPLICATION)
-		if (pool_write_and_flush(SECONDARY(backend), p, len))
+		if (pool_write(SECONDARY(backend), p, len))
 			return POOL_END;
 
 	if (kind == 'P') /* Parse message? */
@@ -2846,6 +2895,15 @@ POOL_STATUS SimpleForwardToBackend(char kind, POOL_CONNECTION *frontend, POOL_CO
 		SimpleForwardToFrontend(kind1, frontend, backend);
 		return pool_flush(frontend);
 	}
+	else
+	{
+		if (pool_flush(MASTER(backend)))
+			return POOL_END;
+
+		if (REPLICATION)
+			if (pool_flush(SECONDARY(backend)))
+				return POOL_END;
+	}
 
 	return POOL_CONTINUE;
 }
@@ -2941,27 +2999,63 @@ static int reset_backend(POOL_CONNECTION_POOL *backend, int qcnt)
 	return 1;
 }
 
+/*
+ * return non 0 if SQL is SELECT statement.
+ */
+static int is_select_query(char *sql)
+{
+	if (pool_config.ignore_leading_white_space)
+	{
+		/* ignore leading white spaces */
+		while (*sql && isspace(*sql))
+			sql++;
+	}
+
+	return (!strncasecmp(sql, "SELECT", 6));
+
+}
+
+/*
+ * return non 0 if SQL is SELECT statement.
+ */
+static int is_sequence_query(char *sql)
+{
+	if (pool_config.ignore_leading_white_space)
+	{
+		/* ignore leading white spaces */
+		while (*sql && isspace(*sql))
+			sql++;
+	}
+
+	if (strncasecmp(sql, "SELECT", 6))
+		return 0;
+
+	sql += 6;
+	while (*sql && isspace(*sql))
+		sql++;
+
+	/* SELECT NEXTVAL('xxx') */
+	if (*sql && !strncasecmp(sql, "NEXTVAL", 7))
+		return 1;
+
+	/* SELECT SETVAL('xxx') */
+	if (*sql && !strncasecmp(sql, "SETVAL", 6))
+		return 1;
+
+	return 0;
+}
+
 /*
  * return non 0 if load balance is possible
  */
 static int load_balance_enabled(POOL_CONNECTION_POOL *backend, char *sql)
 {
-	if (pool_config.load_balance_mode &&
-		DUAL_MODE &&
-		MAJOR(backend) == PROTO_MAJOR_V3 &&
-		TSTATE(backend) == 'I')
-	{
-		if (pool_config.ignore_leading_white_space)
-		{
-			/* ignore leading white spaces */
-			while (*sql && isspace(*sql))
-				sql++;
-		}
-
-		if (!strncasecmp(sql, "SELECT", 6))
-			return 1;
-	}
-	return 0;
+	return (pool_config.load_balance_mode &&
+			DUAL_MODE &&
+			MAJOR(backend) == PROTO_MAJOR_V3 &&
+			TSTATE(backend) == 'I' &&
+			is_select_query(sql) &&
+			!is_sequence_query(sql));
 }
 
 /*
@@ -3429,7 +3523,7 @@ static PreparedStatement *get_prepared_command_portal_and_statement(char *query)
 
 	if (token == NULL)
 	{
-		pool_error("get_prepared_command_portal_and_statement: could not get portal name");
+		pool_debug("get_prepared_command_portal_and_statement: could not get portal name");
 		return NULL;
 	}
 
@@ -3455,7 +3549,7 @@ static PreparedStatement *get_prepared_command_portal_and_statement(char *query)
 
 	if (!*query)
 	{
-		pool_error("get_prepared_command_portal_and_statement: could not get statement");
+		pool_debug("get_prepared_command_portal_and_statement: could not get statement");
 		return NULL;
 	}
 
@@ -3718,7 +3812,8 @@ static void query_ps_status(char *query, POOL_CONNECTION_POOL *backend)
 		return;
 
 	sp = MASTER_CONNECTION(backend)->sp;
-	i = snprintf(psbuf, sizeof(psbuf), "%s %s %s ", sp->database, sp->user, remote_ps_data);
+	i = snprintf(psbuf, sizeof(psbuf), "%s %s %s ",
+				 sp->user, sp->database, remote_ps_data);
 
 	/* skip spaces */
 	while (*query && isspace(*query))
